using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

namespace XELF.Tool {

	public class Layer {
		Texture2D texture;
		Rectangle border;
		Point location;
		bool locked;
		string name = "";
		Vector4[] image;

		public string Name { get { return name; } set { name = value; } }
		public bool Locked { get { return locked; } set { locked = value; } }

		public Point Location {
			get { return location; }
			set {
				location = value;
				UpdateBorder();
			}
		}

		public Texture2D Texture {
			get { return texture; }
			set {
				texture = value;
				UpdateBorder();
				image = new Vector4[texture.Width * texture.Height];
			}
		}
		public void SetImage(GraphicsDevice graphicsDevice, Vector4[] image, Point size) {
			Texture = new Texture2D(
				graphicsDevice, size.X, size.Y, 1, TextureUsage.None, SurfaceFormat.Vector4);
			this.image = image;
			texture.SetData(image);
		}
		public Vector4[] Map() {
			Texture.GetData(image);
			return image;
		}
		public void Unmap() {
			texture.SetData(image);
		}

		void UpdateBorder() {
			border = new Rectangle(location.X, location.Y, texture.Width, texture.Height);
		}

		public Rectangle Border {
			get { return border; }
		}
	}

	public class Document {
		float scale = 8.0f;
		Point translation;
		Matrix transform = Matrix.Identity;
		Matrix inverseTransform = Matrix.Identity;
		string path = "untitled";
		Point size;
		readonly List<Layer> layers = new List<Layer>();

		public Vector2 PointFromScreen(Vector2 screenPosition) {
			return Vector2.Transform(screenPosition, inverseTransform);
		}

		public Point Size {
			get { return size; }
			set { size = value; }
		}
		public Point Translation {
			get { return translation; }
			set {
				translation = value;
				UpdateTransform();
			}
		}

		public List<Layer> Layers {
			get { return layers; }
		}
		public Layer CurrentLayer {
			get { return layers[0]; }
		}
		public Layer PickLayer(ref Point location) {
			for (int i = layers.Count; --i >= 0; ) {
				Layer layer = layers[i];
				if (layer.Border.Contains(location)) {
					location.X -= layer.Location.X;
					location.Y -= layer.Location.Y;
					return layer;
				}
			}
			return null;
		}
		void UpdateTransform() {
			transform = Matrix.CreateScale(scale) * Matrix.CreateTranslation(translation.X, translation.Y, 0);
			inverseTransform = Matrix.Invert(transform);
		}
		public float Scale {
			get { return scale; }
			set {
				scale = value;
				UpdateTransform();
			}
		}

		public void AddLayer(Layer layer) {
			layers.Add(layer);
		}

		public string Path {
			get { return path; }
			set { path = value; }
		}
	}

}
