using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;

namespace XELF.Tool {

	public class TextureFromFile {
		readonly byte[] buffer = new byte[4];

		public static bool ImageFileFormatFromExtension(string extension, out ImageFileFormat format) {
			switch (extension.ToUpper()) {
			default:
				format = ImageFileFormat.Png;
				return false;
			case "PNG":
				format = ImageFileFormat.Png;
				break;
			case "JPG":
			case "JPEG":
				format = ImageFileFormat.Jpg;
				break;
			case "BMP":
				format = ImageFileFormat.Bmp;
				break;
			case "DDS":
				format = ImageFileFormat.Dds;
				break;
			case "DIB":
				format = ImageFileFormat.Dib;
				break;
			case "TGA":
				format = ImageFileFormat.Tga;
				break;
			case "PPM":
				format = ImageFileFormat.Ppm;
				break;
			case "PFM":
				format = ImageFileFormat.Pfm;
				break;
			case "HDR":
				format = ImageFileFormat.Hdr;
				break;
			}
			return true;
		}

		public Texture2D Load(GraphicsDevice device, string fileName) {
			using (System.Drawing.Bitmap bitmap
				= (System.Drawing.Bitmap)System.Drawing.Bitmap.FromFile(fileName)) {
				System.Drawing.Imaging.BitmapData data = bitmap.LockBits(
					new System.Drawing.Rectangle(0, 0, bitmap.Width, bitmap.Height),
					System.Drawing.Imaging.ImageLockMode.ReadOnly,
					 System.Drawing.Imaging.PixelFormat.Format32bppArgb);

				Texture2D texture = new Texture2D(
					device, bitmap.Width, bitmap.Height, 1, TextureUsage.None, SurfaceFormat.Vector4);
				Vector4[] image = new Vector4[bitmap.Width * bitmap.Height];

				long s = data.Scan0.ToInt64();
				int i = 0;
				for (int y = 0; y < bitmap.Height; y++) {
					long r = s + y * data.Stride;
					for (int x = 0; x < bitmap.Width; x++) {
						Marshal.Copy(new IntPtr(r + x * 4), buffer, 0, 4);
						image[i].X = buffer[2] * (1.0f / byte.MaxValue);
						image[i].Y = buffer[1] * (1.0f / byte.MaxValue);
						image[i].Z = buffer[0] * (1.0f / byte.MaxValue);
						image[i].W = buffer[3] * (1.0f / byte.MaxValue);
						i++;
					}
				}
				bitmap.UnlockBits(data);

				texture.SetData(image);
				return texture;
			}
		}
	}

}
